import java.io.*; 
import java.util.Scanner;
import java.util.ArrayList;

/******************************************************************************
*  Sort Blackboard test results back from random question order.
*  
*  Requires CSV file in long-form download "by question and user".
*  Rename this file to simply "downloadlong.csv" for input here.
*  Outputs two files: questions.csv (key), and users.csv (score data).
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2015-02-15
*  @version  1.0
******************************************************************************/

public class BlackboardResultSort {
	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Users (subjects) of the test. */
	ArrayList<User> users;

	/** Questions on the test. */
	ArrayList<Question> questions;
	
	/** Lines from the download file. */
	ArrayList<String> downloadLines;

	/** Download file name. */
	final String DOWNLOAD_FILENAME = "downloadlong.csv";

	/** Questions output file name. */
	final String QUESTIONS_FILENAME = "questions.csv";

	/** Users output file name. */
	final String USERS_FILENAME = "users.csv";

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**	
	*  Constructor.
	*/
	BlackboardResultSort () {
		users = new ArrayList<User>();
		questions = new ArrayList<Question>();
		downloadLines = new ArrayList<String>();
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Entry method.
	*/
	void sortResults() throws IOException {
		readDownloadFile();
		readQuestions();
		readUsers();
		writeQuestionsFile();
		writeUsersFile();
	}

	/**
	*  Read the download file.
	*/
	void readDownloadFile() throws IOException {
		File file = new File(DOWNLOAD_FILENAME);
		Scanner scan = new Scanner(file, "UTF-16");
		while (scan.hasNextLine()) {
			String s = scan.nextLine();
			downloadLines.add(s);
		}
		scan.close();
	}

	/**
	*  Read in the questions.
	*  
	*  We assume that the file starts with all questions for one user,
	*  such that we may ignore duplicated Question IDs later on.
	*/
	void readQuestions() {
		for (int i = 1; i < downloadLines.size(); i++) {
			String[] splitLine = CSVReader.splitLine(downloadLines.get(i));
			String id = splitLine[3];
			Question q = getQuestionById(id);

			// Make new question if not found
			if (q == null) {
				String text = splitLine[4];
				double points = Double.valueOf(splitLine[6]);
				q = new Question(id, text, points);
				questions.add(q);
			}		
		}
	}
	
	/**
	*  Read in the user data.
	*  
	*  Scores are identified by matching question text here
	*  (ignoring source file ID's, since they can vary randomly by user).
	*/
	void readUsers() {
		for (int i = 1; i < downloadLines.size(); i++) {
			String[] splitLine = CSVReader.splitLine(downloadLines.get(i));
			String userName = splitLine[0];
			User u = getUserByName(userName);			
		
			// Make new user if not found
			if (u == null) {
				String lastName = splitLine[1];
				String firstName = splitLine[2];
				int scoreSize = questions.size();				
				u = new User(userName, lastName, firstName, scoreSize);
				users.add(u);
			}
			
			// Add score to this user
			String questionText = splitLine[4];
			Question q = getQuestionByText(questionText);
			int idNumber = q.getIdNumber();
			double score = Double.valueOf(splitLine[7]);
			u.setScore(idNumber-1, score);
		}
	}

	/**
	*  Get question by ID.
	*/
	Question getQuestionById (String id) {
		for (Question q: questions) {
			if (q.getId().equals(id)) {
				return q;
			}
		}
		return null;
	}

	/**
	*  Get question by text.
	*/
	Question getQuestionByText (String text) {
		for (Question q: questions) {
			if (q.getQuestion().equals(text)) {
				return q;
			}
		}
		return null;
	}

	/**
	*  Get user by userName.
	*/
	User getUserByName (String userName) {
		for (User u: users) {
			if (u.getUserName().equals(userName)) {
				return u;
			}
		}
		return null;
	}

	/**
	*  Write the output questions file.
	*/
	void writeQuestionsFile() throws IOException {
		FileWriter fw = new FileWriter(QUESTIONS_FILENAME);
      PrintWriter printOut = new PrintWriter(fw);
		printOut.println(
			makeLineForCSV("Question ID", "Possible Points", "Question"));
		for (Question q: questions) {
			printOut.println(
				makeLineForCSV(q.getId(), String.valueOf(q.getPoints()), q.getQuestion()));
		}	
		printOut.close();
	}

	/**
	*  Write the output users file.
	*/
	void writeUsersFile() throws IOException {

		// Open the file
		FileWriter fw = new FileWriter(USERS_FILENAME);
      PrintWriter printOut = new PrintWriter(fw);

		// Write header
		String header = makeLineForCSV("Username", "Last Name", "First Name");
		for (int i = 0; i < questions.size(); i++) {
			header += ",\"Q" + String.format("%02d", i+1) + "\"";
		}		
		printOut.println(header);

		// Write user data		
		for (User u: users) {
			String data = makeLineForCSV(u.getUserName(), u.getLastName(), u.getFirstName());
			for (int i = 0; i < questions.size(); i++) {
				data += ",\"" + String.valueOf(u.getScore(i)) + "\"";
			}		
			printOut.println(data);
		}

		// Close the file				
		printOut.close();
	}

	/**
	*  Make a CSV line from strings.
	*/
	String makeLineForCSV (String ... list) {
		String s = "";
		for (int i = 0; i < list.length; i++) {
			s += "\"" + list[i] + "\"";
			if (i < list.length-1) {
				s += ",";
			}
		}	
		return s;
	}
	
	/**
	*  Main test function.
	*/
	public static void main (String[] args) throws IOException {
		BlackboardResultSort b = new BlackboardResultSort();
		b.sortResults();		
	}
}

